"""Constraints."""


class ConstraintsError(ValueError):
    """Raised when a constraint is not met."""

    pass


def length_gt_zero(attribute, value):
    if len(value) == 0:
        raise ConstraintsError(
            f"'{attribute}' has to be non-empty. Got {attribute}='{value}'"
        )
    return value


def length_ge(attribute, value, limit: int):
    if len(value) < limit:
        raise ConstraintsError(
            f"'{attribute}' has to have a length of at least {limit}. Got {attribute}='{value}'"
        )
    return value


def no_space_in_str(s: str) -> str:
    if " " in s:
        raise ConstraintsError("whitespace not allowed")
    return s



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['karp',
 'karp.auth',
 'karp.auth.application',
 'karp.auth.application.queries',
 'karp.auth.domain',
 'karp.auth.domain.entities',
 'karp.auth.domain.value_objects',
 'karp.auth_infrastructure',
 'karp.auth_infrastructure.queries',
 'karp.auth_infrastructure.services',
 'karp.cliapp',
 'karp.cliapp.subapps',
 'karp.db_infrastructure',
 'karp.db_infrastructure.types',
 'karp.foundation',
 'karp.foundation.value_objects',
 'karp.lex',
 'karp.lex.application',
 'karp.lex.application.queries',
 'karp.lex.application.repositories',
 'karp.lex.application.use_cases',
 'karp.lex.domain',
 'karp.lex.domain.commands',
 'karp.lex.domain.entities',
 'karp.lex.domain.events',
 'karp.lex.domain.value_objects',
 'karp.lex_infrastructure',
 'karp.lex_infrastructure.queries',
 'karp.lex_infrastructure.repositories',
 'karp.lex_infrastructure.sql',
 'karp.main',
 'karp.main.migrations',
 'karp.main.migrations.versions',
 'karp.search',
 'karp.search.application',
 'karp.search.application.queries',
 'karp.search.application.repositories',
 'karp.search.application.transformers',
 'karp.search.domain',
 'karp.search.domain.query_dsl',
 'karp.search_infrastructure',
 'karp.search_infrastructure.elasticsearch6',
 'karp.search_infrastructure.queries',
 'karp.search_infrastructure.repositories',
 'karp.search_infrastructure.transformers',
 'karp.tests',
 'karp.tests.e2e',
 'karp.tests.factories',
 'karp.tests.foundation',
 'karp.tests.foundation.unit',
 'karp.tests.integration',
 'karp.tests.integration.auth',
 'karp.tests.unit',
 'karp.tests.unit.auth',
 'karp.tests.unit.auth_infrastructure',
 'karp.tests.unit.lex',
 'karp.tests.unit.lex.value_objects',
 'karp.tests.unit.lex_infrastructure',
 'karp.tests.unit.search',
 'karp.tests.unit.search_infrastructure',
 'karp.util',
 'karp.util.convert',
 'karp.utility',
 'karp.webapp',
 'karp.webapp.contrib',
 'karp.webapp.dependencies',
 'karp.webapp.routes']

package_data = \
{'': ['*'],
 'karp': ['data_files/*', 'schema/*'],
 'karp.tests': ['data/*', 'data/config/*']}

install_requires = \
['Deprecated>=1.2.13,<2.0.0',
 'TatSu>=5.8.3,<6.0.0',
 'alembic>=1.8.1,<2.0.0',
 'asgi-correlation-id>=3.0.1,<4.0.0',
 'elasticsearch-dsl>=6,<7',
 'elasticsearch>=6,<7',
 'environs>=9.3.4,<10.0.0',
 'fastapi>=0.88.0,<0.89.0',
 'injector>=0.20.1,<0.21.0',
 'json-streams>=0.12.0,<0.13.0',
 'paradigmextract>=0.1.1,<0.2.0',
 'pydantic>=1.10.2,<2.0.0',
 'pyjwt[crypto]>=2.6.0,<3.0.0',
 'python-dotenv>=0.19.0,<0.20.0',
 'python-json-logger>=2.0.4,<3.0.0',
 'regex>=2022.8.17,<2023.0.0',
 'sb-json-tools>=0.9.1,<0.10.0',
 'sqlalchemy-json>=0.5.0,<0.6.0',
 'sqlalchemy-utils>=0.38.3,<0.39.0',
 'sqlalchemy>=1.4.44,<2.0.0',
 'tabulate>=0.9.0,<0.10.0',
 'tenacity>=8.0.1,<9.0.0',
 'tqdm>=4.64.1,<5.0.0',
 'typer>=0.7.0,<0.8.0',
 'ulid-py>=1.1.0,<2.0.0',
 'urllib3>=1.26.13,<2.0.0']

extras_require = \
{'mysql': ['PyMySQL>=1.0.2,<2.0.0',
           'aiomysql>=0.1.1,<0.2.0',
           'mysqlclient>=2.1.1,<3.0.0'],
 'sqlite': ['aiosqlite>=0.17.0,<0.18.0']}

entry_points = \
{'console_scripts': ['karp-cli = karp.cliapp.main:cliapp'],
 'karp.infrastructure': ['es6 = karp.infrastructure.elasticsearch6',
                         'jwt = karp.infrastructure.jwt',
                         'sql = karp.infrastructure.sql',
                         'testing = karp.infrastructure.testing']}

setup_kwargs = {
    'name': 'karp-backend',
    'version': '6.1.12',
    'description': 'Karp backend',
    'long_description': "# Karp backend\n\n[![PyPI version](https://badge.fury.io/py/karp-backend.svg)](https://badge.fury.io/py/karp-backend)\n[![Build Status](https://github.com/spraakbanken/karp-backend/workflows/Build/badge.svg)](https://github.com/spraakbanken/karp-backend/actions)\n[![CodeScene Code Health](https://codescene.io/projects/24151/status-badges/code-health)](https://codescene.io/projects/24151)\n[![codecov](https://codecov.io/gh/spraakbanken/karp-backend/branch/main/graph/badge.svg?token=iwTQnHKOpm)](https://codecov.io/gh/spraakbanken/karp-backend)\n\nThis in the version 6 of Karp backend, [for the legacy version (v5)](https://github.com/spraakbanken/karp-backend-v5).\n\n## Setup\n\nThis project uses [poetry](https://python-poetry.org) and\n[MariaDB](https://mariadb.org/).\n\n1. Run `make install` or `make install-dev` for a develop-install\n2. Install MariaDB and create a database\n3. Setup environment variables (can be placed in a `.env` file in the root and then **?** `poetry run` sets those):\n   ```\n   export DB_DATABASE=<name of database>\n   export DB_USER=<database user>\n   export DB_PASSWORD=<user's password>\n   export DB_HOST=localhost\n   export AUTH_JWT_PUBKEY_PATH=/path/to/pubkey\n   ```\n4. Activate the virtual environment by running: `poetry shell`\n5. Run `karp-cli db up` to initialize database\n6. Run `make serve` or `make serve-w-reload` to start development server\n\n   or `poetry shell` and then `uvicorn asgi:app`\n\n7. To setup Elasticsearch, download Elasticsearch 6.x and start it\n8. Install elasticsearch python libs for the right version\n   1. If you use Elasticsearch 6.x, run `source <VENV_NAME>/bin/activate` and `pip install -e .[elasticsearch6]`\n9. Add environment variables\n\n```\nexport ES_ENABLED=true\nexport ELASTICSEARCH_HOST=localhost:9200\nexport SEARCH_CONTEXT=es6_search_service\n```\n\n## Create test resources\n\n1. `poetry shell` and then:\n2. `karp-cli entry-repo create karp/tests/data/config/places.json`\n3. `karp-cli resource create karp/tests/data/config/places.json`\n4. `karp-cli entries add places tests/data/places.jsonl`\n5. Do the same for `municipalities`\n6. `karp-cli resource publish places`\n7. `karp-cli resource publish municipalities`\n\n## Pre-processing data before publishing\n\n** TODO: review this **\nCan be used to have less downtime, because sometimes the preprocessing may\nbe faster on another machine than the machine that talks to Elasticsearch.\nDo `create` and `import` on both machines, with the same data. Use\nmachine 1 to preprocess and use result on machine 2.\n\n1. Create resource and import data as usual.\n2. Run `karp-cli preprocess --resource_id places --version 2 --filename places_preprocessed`\n\n   `places_preprocessed` will contain a pickled dataset containing everything that is needed\n\n3. Run `karp-cli publish_preprocessed --resource_id places --version 2 --data places_preprocessed`\n4. Alternatively run `karp-cli reindex_preprocessed --resource_id places --data places_preprocessed`\n   , if the resource was already published.\n\n## Technologies\n\n### Python\n\n- Poetry >= 3.10\n- FastAPI\n- SQLAlchemy\n- Typer\n- Elasticsearch\n- Elasticsearch DSL\n\n### Databases\n\n- MariaDB\n- Elasticsearch\n\n## Development\n\n### Version handling\n\nVersion can be bumped with [`bump2version`](https://pypi.org/project/bump2version/).\n\nUsage:\n\n- Increase patch number `a.b.X => a.b.(X+1)`: `make bumpversion` or `bumpversion patch`\n- Increase minor number `a.X.c => a.(X+1).0`: `make bumpversion-minor` or `bumpversion minor`\n- Increase major number `X.b.c => (X+1).0.0`: `make bumpversion-major` or `bumpversion major`\n- To custom version `a.b.c => X.Y.Z`: `bumpversion --new-version X.Y.Z`\n\n`bumpversion` is configured in [`.bumpversion.cfg`](.bumpversion.cfg).\n\nThe version is changed in the following files:\n\n- [`setup.py`](setup.py)\n- [`src/karp/__init__.py`](src/karp/__init__.py)\n- [`.bumpversion.cfg`](.bumpversion.cfg)\n- [`doc/karp_api_spec.yaml`](doc/karp_api_spec.yaml)\n",
    'author': 'Språkbanken at the University of Gothenburg',
    'author_email': 'sb-info@svenska.gu.se',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://spraakbanken.gu.se',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



from typing import Dict, Optional

import pydantic

from karp.foundation.value_objects import UniqueId, make_unique_id

from .base import Command


class CreateEntryRepository(Command):
    entity_id: UniqueId = pydantic.Field(default_factory=make_unique_id)
    repository_type: str
    name: str
    connection_str: Optional[str] = None
    config: Dict
    message: str
    user: str

    @classmethod
    def from_dict(cls, data: Dict, *, user: str, message: Optional[str] = None):
        return cls(
            entity_id=make_unique_id(),
            repository_type=data.pop("repository_type", "default"),
            name=data.pop("resource_id"),
            connection_str=data.pop("connection_str", None),
            config=data,
            user=user,
            message=message or "Entry repository created",
        )


class DeleteEntryRepository(Command):
    entity_id: UniqueId
    message: str
    user: str



"""Unit tests for JWTAuthenticator"""
from datetime import datetime, timedelta
from pathlib import Path
from typing import Dict, Optional, Type

import jwt
import pydantic
import pytest

from karp.foundation.time import utc_now
from karp.auth.domain.errors import (
    AuthError,
    ExpiredToken,
    InvalidTokenSignature,
    InvalidTokenAudience,
    InvalidTokenPayload,
    TokenError,
)
from karp.auth_infrastructure.services.jwt_auth_service import (
    JWTAuthService,
    JWTCreds,
    JWTMeta,
    JWTPayload,
)

from karp.main.config import AUTH_JWT_AUDIENCE
from karp.tests.unit.auth import adapters
from karp.tests.integration.auth.adapters import create_access_token
from cryptography.hazmat.primitives.asymmetric import rsa

# Generate our key

other_key = rsa.generate_private_key(
    public_exponent=65537,
    key_size=2048,
)


@pytest.fixture
def jwt_authenticator():
    return JWTAuthService(
        pubkey_path=Path("karp/tests/data/pubkey.pem"),
        is_resource_protected=adapters.InMemoryIsResourceProtected(),
    )


def test_authenticate_invalid_token(jwt_authenticator):
    with pytest.raises(AuthError):
        jwt_authenticator.authenticate("scheme", "invalid")


def test_authenticate_expired_token(jwt_authenticator):
    token = create_access_token(
        user=None,
        levels={},
        expires_in=-1,
    )

    with pytest.raises(ExpiredToken):
        jwt_authenticator.authenticate("scheme", token)


class TestAuthTokens:
    def test_can_create_access_token_successfully(self, jwt_authenticator) -> None:
        access_token = create_access_token(
            user="test_user",
            levels={},
        )
        user = jwt_authenticator.authenticate("bearer", access_token)
        assert user is not None
        assert user.identifier == "test_user"

    def test_token_missing_user_is_invalid(self, jwt_authenticator) -> None:
        access_token = create_access_token(
            user=None,
            levels={},
        )
        with pytest.raises(TokenError):
            jwt_authenticator.authenticate("bearer", access_token)

    @pytest.mark.parametrize(
        "user, levels, secret_key, jwt_audience, exception",
        (
            ("user", {}, other_key, AUTH_JWT_AUDIENCE, TokenError),
            ("user", None, None, AUTH_JWT_AUDIENCE, InvalidTokenPayload),
            pytest.param(
                "user",
                {},
                None,
                "othersite:auth",
                InvalidTokenAudience,
                marks=pytest.mark.xfail(reason="audience not impl on auth"),
            ),
            (None, {}, None, AUTH_JWT_AUDIENCE, InvalidTokenPayload),
        ),
    )
    def test_invalid_token_content_raises_error(
        self,
        jwt_authenticator,
        user: str,
        levels: Optional[Dict],
        secret_key: Optional[str],
        jwt_audience: str,
        exception: Type[BaseException],
    ) -> None:
        with pytest.raises(exception):
            access_token = create_access_token(
                user=user,
                levels=levels,
                priv_key=secret_key,
                audience=jwt_audience,
            )
            jwt_authenticator.authenticate("bearer", access_token)



import logging

from karp.search.application.repositories import (
    Index,
    IndexUnitOfWork,
)


logger = logging.getLogger("karp")


class NoOpIndex(Index):
    pass

    def create_index(self, *args, **kwargs):
        logger.debug("create_index called with args=%s, kwargs=%s", args, kwargs)

    def publish_index(self, *args, **kwargs):
        logger.debug("publish_index called with args=%s, kwargs=%s", args, kwargs)

    def add_entries(self, *args, **kwargs):
        logger.debug("add_entries called with args=%s, kwargs=%s", args, kwargs)

    def delete_entry(self, *args, **kwargs):
        logger.debug("delete_entry called with args=%s, kwargs=%s", args, kwargs)


class NoOpIndexUnitOfWork(IndexUnitOfWork):
    def __init__(self):
        self._repo = NoOpIndex()

    @property
    def repo(self):
        return self._repo

    def _commit(self, *args, **kwargs):
        logger.debug("_commit called with args=%s, kwargs=%s", args, kwargs)

    def _close(self, *args, **kwargs):
        logger.debug("_close called with args=%s, kwargs=%s", args, kwargs)

    def rollback(self, *args, **kwargs):
        logger.debug("rollback called with args=%s, kwargs=%s", args, kwargs)



import pytest

from karp.lex_infrastructure.repositories.sql_entries import SqlEntryRepository

from karp.tests.unit.lex import factories
from karp.utility.time import utc_now


@pytest.fixture(name="entry_repo")
def fixture_entry_repo(sqlite_session_factory):
    session = sqlite_session_factory()
    entry_repo = SqlEntryRepository.from_dict(
        name="test_name",
        resource_config={},
        session=session,
    )
    yield entry_repo

    entry_repo.teardown()


@pytest.fixture(name="entry_repo2")
def fixture_entry_repo2(sqlite_session_factory):
    session = sqlite_session_factory()
    entry_repo = SqlEntryRepository.from_dict(
        name="test_name2",
        resource_config={},
        session=session,
    )
    yield entry_repo

    entry_repo.teardown()


def test_create_entry_repository(entry_repo):
    assert entry_repo.entity_ids() == []


def test_save_entry_to_entry_repo(entry_repo):
    entry = factories.EntryFactory()
    entry_repo.save(entry)

    assert entry_repo.by_id(entry.id).version == entry.version


def test_update_entry_to_entry_repo(entry_repo):
    entry = factories.EntryFactory()
    entry_repo.save(entry)

    entry.update_body({"body": "changed"}, user="kristoff@example.com", message="hi")
    entry_repo.save(entry)

    assert entry_repo.by_id(entry.id).version == entry.version
    assert entry_repo.by_id(entry.id, version=1).version == 1


def test_discard_entry_and_insert_new(entry_repo):
    entry = factories.EntryFactory(last_modified=utc_now() - 3600.0, body={"a": "b"})
    entry_repo.save(entry)

    entry.discard(
        user="kristoff@example.com",
        message="hi",
    )
    assert entry.discarded
    entry_repo.save(entry)

    assert entry_repo.entity_ids() == []

    entry2 = factories.EntryFactory(body={"a": "b"})
    entry_repo.save(entry2)

    assert entry_repo.entity_ids() == [entry2.entity_id]


def test_create_entry_repository2(entry_repo2):
    assert entry_repo2.entity_ids() == []


class TestSqlEntryRepoGetHistory:
    def test_discard_entry_and_insert_new(self, entry_repo):
        pass


class TestSqlEntryRepoByReferencable:
    def test_by_referenceable_wo_filter_raises_value_error(self, entry_repo):
        with pytest.raises(ValueError):
            entry_repo.by_referenceable()

    def test_by_referenceable_w_kwargs_returns_entry(self, entry_repo):
        entry = factories.EntryFactory(body={"a": "b"})
        entry_repo.save(entry)

        entry_copies = entry_repo.by_referenceable(a="b")

        print(f"{entry_copies=}")
        assert entry_copies[0].id == entry.id

    def test_by_referenceable_w_filters_returns_entry(self, entry_repo):
        entry = factories.EntryFactory(body={"a": "b"})
        entry_repo.save(entry)

        entry_repo._session.commit()
        entry_copies = entry_repo.by_referenceable(filters={"a": "b"})

        print(f"{entry_copies=}")
        assert entry_copies[0].id == entry.id

        entry.update_body({"a": "b", "c": "d"}, user="user")
        entry_repo.save(entry)

        entry_copies = entry_repo.by_referenceable(filters={"a": "b"})

        assert len(entry_copies) == 1
        assert entry_copies[0].version == 2

    def test_by_referenceable_w_many_filters_returns_one_entry(self, entry_repo):
        entry = factories.EntryFactory(body={"a": "b"})
        entry_repo.save(entry)
        entry2 = factories.EntryFactory(body={"a": "b", "c": "d"})
        entry_repo.save(entry2)

        entry_repo._session.commit()
        entry_copies = entry_repo.by_referenceable(filters={"a": "b", "c": "d"})

        print(f"{entry_copies=}")
        assert entry_copies[0].id == entry2.id

        assert len(entry_copies) == 1



import pytest
import pydantic

from karp.lex.domain.value_objects import ResourceConfig


class TestResourceConfig:
    def test_invalid_input_raises(self):
        with pytest.raises(pydantic.ValidationError):
            ResourceConfig(fields=3)



from .entries import (
    GetEntryDiff,
    GetEntryHistory,
    GetHistory,
    GetHistoryDto,
    EntryDiffDto,
    EntryDto,
    EntryViews,
    EntryDiffRequest,
    EntryHistoryRequest,
    HistoryDto,
)
from .entry_repos import ListEntryRepos, EntryRepoDto, ReadOnlyEntryRepoRepositry
from .resources import (
    GetPublishedResources,
    ResourceDto,
    GetResources,
    GetEntryRepositoryId,
    ReadOnlyResourceRepository,
)
from .network import GetReferencedEntries, ReferenceDto



import re
import typing
from typing import Union

import elasticsearch_dsl as es_dsl

from karp.search.application.queries import QueryRequest
from karp.search.domain.errors import IncompleteQuery, UnsupportedQuery
from karp.search.domain.query import Query
from karp.search.domain.query_dsl import basic_ast as ast
from karp.search.domain.query_dsl import is_a, op


class EsQuery(Query):
    query: typing.Optional[es_dsl.query.Query] = None
    resource_str: typing.Optional[str] = None

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        # self.query = None
        # self.resource_str: Optional[str] = None

    def parse_arguments(self, args, resource_str: str):
        super().parse_arguments(args, resource_str)
        self.resource_str = resource_str
        # if not self.ast.is_empty():
        #     self.query = create_es_query(self.ast.root)

    def _self_name(self) -> str:
        return f"EsQuery query={self.query} resource_str={self.resource_str}"

    @classmethod
    def from_query_request(cls, request: QueryRequest):
        query = cls(fields=[], resources=request.resource_ids, sort=[])
        query.from_ = request.from_
        query.size = request.size
        query.lexicon_stats = request.lexicon_stats
        query.q = request.q or ""
        query.sort = request.sort
        return query

    class Config:
        arbitrary_types_allowed = True



from karp.lex.domain.value_objects.entry_schema import EntrySchema
import pytest

from karp.utility.json_schema import create_entry_json_schema

CONFIG_PLACES = """{
  "resource_id": "places",
  "resource_name": "Platser i Sverige",
  "fields": {
    "name": {
      "type": "string",
      "required": True
    },
    "municipality": {
      "collection": True,
      "type": "number",
      "required": True
    },
    "population": {
      "type": "number"
    },
    "area": {
      "type": "number"
    },
    "density": {
      "type": "number"
    },
    "code": {
      "type": "number",
      "required": True
    }
  },
  "sort": "name",
  "id": "code"
}"""


@pytest.fixture()
def problem_config() -> dict:
    return {
        "resource_id": "konstruktikon",
        "resource_name": "Konstruktikon",
        "fields": {
            "cat": {"type": "string"},
            "illustration": {"type": "string"},
            "cee": {"type": "string", "collection": True},
            "coll": {"type": "string", "collection": True},
            "createdBy": {"type": "string"},
            "definition": {"type": "string"},
            "examples": {"type": "string", "collection": True},
            "entryStatus": {"type": "string"},
            "intConstElem": {
                "type": "object",
                "fields": {
                    "role": {"type": "string"},
                    "name": {"type": "string"},
                    "cat": {"type": "string"},
                    "lu": {"type": "string"},
                    "gfunc": {"type": "string"},
                    "msd": {"type": "string"},
                    "aux": {"type": "string"},
                },
                "collection": True,
            },
            "extConstElem": {
                "type": "object",
                "fields": {
                    "role": {"type": "string"},
                    "name": {"type": "string"},
                    "cat": {"type": "string"},
                    "lu": {"type": "string"},
                    "gfunc": {"type": "string"},
                    "msd": {"type": "string"},
                    "aux": {"type": "string"},
                },
                "collection": True,
            },
            "inheritance": {"type": "string", "collection": True},
            "type": {"type": "string", "collection": True},
            "structure": {"type": "string", "collection": True},
            "constructionID": {"type": "string", "required": True},
            "BCxnID": {"type": "string"},
            "evokes": {"type": "string", "collection": True},
            "comment": {"type": "string"},
            "reference": {"type": "string"},
            "internal_comment": {"type": "string"},
        },
        "sort": "constructionID",
        "protected": {"read": False},
        "id": "constructionID",
    }


def test_error(problem_config: dict):
    json_schema = create_entry_json_schema(problem_config["fields"])
    _entry_schema = EntrySchema(json_schema)


def test_create_json_schema(json_schema_config):
    json_schema = create_entry_json_schema(json_schema_config["fields"])
    assert json_schema["type"] == "object"


def test_create_complex_json_schema():
    config = {
        "fields": {
            "id": {"type": "string", "required": True},
            "s_nr": {"type": "integer"},
            "ortografi": {"type": "string", "required": True},
            "ordklass": {"type": "string", "required": True},
            "böjningsklass": {"type": "string", "required": True},
            "artikelkommentar": {"type": "string"},
            "ursprung": {"type": "string", "collection": True},
            "SOLemman": {
                "type": "object",
                "collection": True,
                "fields": {
                    "l_nr": {"type": "integer", "required": True},
                    "s_nr": {"type": "integer", "required": True},
                    "lm_sabob": {"type": "integer", "required": True},
                    "ortografi": {"type": "string", "required": True},
                    "lemmatyp": {"type": "string"},
                    "lemmaundertyp": {"type": "string"},
                    "stam": {"type": "string"},
                    "böjning": {"type": "string"},
                    "ordbildning": {"type": "string"},
                    "analys": {"type": "string"},
                    "sorteringsform": {"type": "string"},
                    "kommentar": {"type": "string"},
                    "tagg": {"type": "string"},
                    "ursprung": {"type": "string"},
                    "betydelser": {
                        "type": "object",
                        "collection": True,
                        "required": True,
                        "fields": {
                            "x_nr": {"type": "integer", "required": True},
                            "status": {"type": "string"},
                            "etymologier": {
                                "type": "object",
                                "collection": True,
                                "fields": {
                                    "förstaBelägg": {"type": "string"},
                                    "källa": {"type": "string"},
                                    "beskrivning": {"type": "string"},
                                    "status": {"type": "integer"},
                                    "kommentar": {"type": "string"},
                                },
                            },
                            "kc_nr": {"type": "integer", "required": True},
                            "huvudkommentar": {"type": "string"},
                            "formellKommmentar": {"type": "string"},
                            "formellKommmentarExempel": {"type": "string"},
                            "formellKommmentarTillägg": {"type": "string"},
                            "definition": {"type": "string"},
                            "definitionstillägg": {"type": "string"},
                            "slutkommentar": {"type": "string"},
                            "ämnesområden": {
                                "type": "object",
                                "collection": True,
                                "fields": {
                                    "ämne": {"type": "string"},
                                    "specifikt": {"type": "string"},
                                },
                            },
                            "hänvisningar": {
                                "type": "object",
                                "collection": True,
                                "fields": {
                                    "kc_nr": {"type": "integer"},
                                    "l_nr": {"type": "integer"},
                                    "typ": {"type": "string"},
                                    "hänvisning": {"type": "string"},
                                    "kommentar": {"type": "string"},
                                    "status": {"type": "integer"},
                                    "visas": {"type": "boolean"},
                                },
                            },
                            "morfex": {
                                "type": "object",
                                "collection": True,
                                "fields": {
                                    "ortografi": {"type": "string"},
                                    "hänvisning": {"type": "string"},
                                    "kommentar": {"type": "string"},
                                    "visas": {"type": "boolean"},
                                },
                            },
                            "syntex": {
                                "type": "object",
                                "collection": True,
                                "fields": {
                                    "typ": {"type": "string"},
                                    "text": {"type": "string"},
                                    "kommentar": {"type": "string"},
                                    "visas": {"type": "boolean"},
                                },
                            },
                            "valenser": {
                                "type": "object",
                                "collection": True,
                                "fields": {
                                    "vl_nr": {"type": "integer"},
                                    "typ": {"type": "string"},
                                    "prevalens": {"type": "string"},
                                    "beskrivning": {"type": "string"},
                                    "kommentar": {"type": "string"},
                                    "status": {"type": "integer"},
                                    "visas": {"type": "boolean"},
                                },
                            },
                            "underbetydelser": {
                                "type": "object",
                                "collection": True,
                                "fields": {
                                    "kc_nr": {"type": "integer", "required": True},
                                    "huvudkommentar": {"type": "string"},
                                    "typ": {"type": "string"},
                                    "formellKommmentar": {"type": "string"},
                                    "formellKommmentarvar": {"type": "string"},
                                    "formellKommmentarandra": {"type": "string"},
                                    "defintion": {"type": "string"},
                                    "definitionstillägg": {"type": "string"},
                                    "slutkommentar": {"type": "string"},
                                    "ämnesområden": {
                                        "type": "object",
                                        "collection": True,
                                        "fields": {
                                            "ämne": {"type": "string"},
                                            "specifikt": {"type": "string"},
                                        },
                                    },
                                    "hänvisningar": {
                                        "type": "object",
                                        "collection": True,
                                        "fields": {
                                            "kc_nr": {"type": "integer"},
                                            "l_nr": {"type": "integer"},
                                            "typ": {"type": "string"},
                                            "hänvisning": {"type": "string"},
                                            "kommentar": {"type": "string"},
                                            "status": {"type": "integer"},
                                            "visas": {"type": "boolean"},
                                        },
                                    },
                                    "morfex": {
                                        "type": "object",
                                        "collection": True,
                                        "fields": {
                                            "ortografi": {"type": "string"},
                                            "hänvisning": {"type": "string"},
                                            "kommentar": {"type": "string"},
                                            "visas": {"type": "boolean"},
                                        },
                                    },
                                    "syntex": {
                                        "type": "object",
                                        "collection": True,
                                        "fields": {
                                            "typ": {"type": "string"},
                                            "text": {"type": "string"},
                                            "kommentar": {"type": "string"},
                                            "visas": {"type": "boolean"},
                                        },
                                    },
                                    "valenser": {
                                        "type": "object",
                                        "collection": True,
                                        "fields": {
                                            "vl_nr": {"type": "integer"},
                                            "typ": {"type": "string"},
                                            "prevalens": {"type": "string"},
                                            "beskrivning": {"type": "string"},
                                            "kommentar": {"type": "string"},
                                            "status": {"type": "integer"},
                                            "visas": {"type": "boolean"},
                                        },
                                    },
                                },
                            },
                            "idiom": {
                                "type": "object",
                                "collection": True,
                                "fields": {
                                    "i_nr": {"type": "integer"},
                                    "hänvisning": {"type": "integer"},
                                    "idiom": {"type": "string"},
                                    "formellKommmentar": {"type": "string"},
                                    "alternativinledare": {"type": "string"},
                                    "alternativform": {"type": "string"},
                                    "status": {"type": "integer"},
                                    "idiombetydelser": {
                                        "type": "object",
                                        "collection": True,
                                        "fields": {
                                            "ix_nr": {"type": "integer"},
                                            "definitionsinledare": {"type": "string"},
                                            "huvudkommentar": {"type": "string"},
                                            "definition": {"type": "string"},
                                            "definitionstillägg": {"type": "string"},
                                            "exempel": {"type": "string"},
                                            "status": {"type": "integer"},
                                            "visas": {"type": "boolean"},
                                        },
                                    },
                                    "visas": {"type": "boolean"},
                                },
                            },
                        },
                    },
                    "sentenserOchStilrutor": {
                        "type": "object",
                        "collection": True,
                        "fields": {
                            "typ": {"type": "string"},
                            "text": {"type": "string"},
                            "origid": {"type": "integer"},
                            "origOrd": {"type": "string"},
                            "visas": {"type": "boolean"},
                        },
                    },
                    "uttal": {
                        "type": "object",
                        "collection": True,
                        "fields": {
                            "visas": {"type": "boolean"},
                            "typ": {"type": "string"},
                            "lemmaMedTryckangivelse": {"type": "string"},
                            "fonetikparentes": {"type": "string"},
                            "fonetikkommentar": {"type": "string"},
                            "filenamInlästUttal": {"type": "string"},
                        },
                    },
                    "lemma_referenser": {
                        "type": "object",
                        "collection": True,
                        "fields": {
                            "l_nr": {"type": "integer", "required": True},
                            "lm_sabob": {"type": "integer", "required": True},
                            "ortografi": {"type": "string", "required": True},
                            "lemmatyp": {"type": "string"},
                            "lemmaundertyp": {"type": "string"},
                            "stam": {"type": "string"},
                            "böjning": {"type": "string"},
                            "ordbildning": {"type": "string"},
                            "analys": {"type": "string"},
                            "sorteringsform": {"type": "string"},
                            "kommentar": {"type": "string"},
                            "tagg": {"type": "string"},
                            "ursprung": {"type": "string"},
                        },
                    },
                },
            },
            "SAOLLemman": {
                "type": "object",
                "collection": True,
                "fields": {
                    "lemmaId": {"type": "string"},
                    "ortografi": {"type": "string", "required": True},
                    "homografNr": {
                        "type": "object",
                        "fields": {
                            "nummer": {"type": "string"},
                            "version": {"type": "string"},
                        },
                    },
                    "fonetik": {
                        "type": "object",
                        "collection": True,
                        "fields": {
                            "form": {"type": "string"},
                            "kommentar": {"type": "string"},
                        },
                    },
                    "saolKlass": {"type": "string"},
                    "analys": {"type": "string"},
                    "böjning": {"type": "string"},
                    "alt": {
                        "type": "object",
                        "collection": True,
                        "fields": {
                            "grundform": {"type": "string"},
                            "homografNr": {
                                "type": "object",
                                "fields": {
                                    "nummer": {"type": "string"},
                                    "version": {"type": "string"},
                                },
                            },
                            "analys": {"type": "string"},
                            "typ": {"type": "string"},
                        },
                    },
                    "fack": {"type": "string", "collection": True},
                    "kommentarTillOrdklassSAOL11": {"type": "string"},
                    "saol_status": {"type": "integer"},
                    "ursprung": {"type": "string"},
                    "betydelser": {
                        "type": "object",
                        "collection": True,
                        "fields": {
                            "id": {"type": "string"},
                            "ordningSAOL14": {"type": "string"},
                            "definition": {"type": "string"},
                            "exempel": {
                                "type": "object",
                                "collection": True,
                                "fields": {
                                    "text": {"type": "string"},
                                    "parafras": {"type": "string"},
                                },
                            },
                            "huvudkommentar": {"type": "string"},
                            "formellKommentar": {"type": "string", "collection": True},
                        },
                    },
                    "relation": {
                        "type": "object",
                        "collection": True,
                        "fields": {
                            "typ": {"type": "string"},
                            "till_id": {"type": "string"},
                        },
                    },
                    "kommentar": {"type": "string"},
                    "ptv": {"type": "string"},
                    "smdb_n": {"type": "string"},
                    "hänvisningar": {
                        "type": "object",
                        "collection": True,
                        "fields": {
                            "typ": {"type": "string"},
                            "till_id": {"type": "string"},
                        },
                    },
                },
            },
        }
    }
    _json_schema = create_entry_json_schema(config["fields"])


class TestCreateJsonSchema:
    @pytest.mark.parametrize("field_type", ["long_string"])
    def test_create_with_type(self, field_type: str):
        resource_config = {"field_name": {"type": field_type}}
        json_schema = create_entry_json_schema(resource_config)
        _entry_schema = EntrySchema(json_schema)



"""Factories used in tests."""
import typing

import factory
import factory.fuzzy
from faker.providers import BaseProvider

from karp.lex.domain import commands, entities, events
from karp.foundation.value_objects import make_unique_id, MachineName
from karp.utility.time import utc_now


class ResourceConfigProvider(BaseProvider):
    def resource_config(self) -> typing.Dict:
        return {
            "sort": ["baseform"],
            "fields": {
                "baseform": {"type": "string", "required": True},
                "wordclass": {"type": "string"},
            },
            "id": "baseform",
        }


class MachineNameProvider(BaseProvider):
    def machine_name(self) -> MachineName:
        return MachineName(name=factory.fuzzy.FuzzyText(length=6, prefix="resource_"))


factory.Faker.add_provider(MachineNameProvider)
factory.Faker.add_provider(ResourceConfigProvider)


class ResourceFactory(factory.Factory):
    class Meta:
        model = entities.Resource

    entity_id = factory.LazyFunction(make_unique_id)
    entry_repo_id = factory.LazyFunction(make_unique_id)
    resource_id: str = factory.Faker("word")
    name: str = factory.Faker("word")
    config: str = factory.Faker("resource_config")
    last_modified_by: str = factory.Faker("email")
    last_modified = factory.LazyFunction(utc_now)
    message = "resource created"


class EntryFactory(factory.Factory):
    class Meta:
        model = entities.Entry

    entity_id = factory.LazyFunction(make_unique_id)
    # entry_id = factory.Faker("word")
    last_modified_by = factory.Faker("email")
    last_modified = factory.LazyFunction(utc_now)
    message = "entry created"
    body = {}
    repository_id = factory.LazyFunction(make_unique_id)


def random_resource(config: typing.Optional[typing.Dict] = None):
    config = config or {"fields": {"wf": {"type" "string"}, "id": "wf"}}
    return entities.create_resource(
        entity_id=make_unique_id(),
        entry_repo_id=make_unique_id(),
        resource_id="resource",
        config=config,
        message="Resource add",
        created_by="kristoff@example.com",
    )


class ResourceCreatedFactory(factory.Factory):
    class Meta:
        model = events.ResourceCreated

    timestamp = factory.LazyFunction(utc_now)
    entity_id = factory.LazyFunction(make_unique_id)
    entry_repo_id = factory.LazyFunction(make_unique_id)
    resource_id = factory.Faker("word")
    name = factory.Faker("word")
    config = factory.Faker("resource_config")
    user = factory.Faker("email")
    message = "resource created"


class ResourcePublishedFactory(factory.Factory):
    class Meta:
        model = events.ResourcePublished

    timestamp = factory.LazyFunction(utc_now)
    entity_id = factory.LazyFunction(make_unique_id)
    entry_repo_id = factory.LazyFunction(make_unique_id)
    resource_id = factory.Faker("word")
    name = factory.Faker("word")
    config = factory.Faker("resource_config")
    user = factory.Faker("email")
    version = factory.Sequence(int)
    message = "resource created"


class EntryAddedFactory(factory.Factory):
    class Meta:
        model = events.EntryAdded

    timestamp = factory.LazyFunction(utc_now)
    id = factory.LazyFunction(make_unique_id)
    resource_id = factory.Faker("word")
    entry_id = factory.Faker("word")
    body = factory.Faker("resource_config")
    user = factory.Faker("email")
    message = "resource created"


class CreateEntryRepositoryFactory(factory.Factory):
    class Meta:
        model = commands.CreateEntryRepository

    entity_id = factory.LazyFunction(make_unique_id)
    name = factory.Faker("word")
    repository_type = "fake"
    config = factory.Faker("resource_config")
    message = "entry repository created"
    user = factory.Faker("email")


class DeleteEntryRepositoryFactory(factory.Factory):
    class Meta:
        model = commands.DeleteEntryRepository

    entity_id = factory.LazyFunction(make_unique_id)
    message = "entry repository deleted"
    user = factory.Faker("email")


class MachineNameFactory(factory.Factory):
    class Meta:
        model = MachineName

    name = factory.Faker("word")


class CreateResourceFactory(factory.Factory):
    class Meta:
        model = commands.CreateResource

    entity_id = factory.LazyFunction(make_unique_id)
    # resource_id = factory.SubFactory(MachineNameFactory)
    resource_id = factory.fuzzy.FuzzyText(length=6, prefix="resource_")
    name = factory.Faker("word")
    repository_type = "fake"
    config = factory.Faker("resource_config")
    entry_repo_id = factory.LazyFunction(make_unique_id)
    message = "created"
    user = factory.Faker("email")


class UpdateResourceFactory(factory.Factory):
    class Meta:
        model = commands.UpdateResource

    resource_id = factory.Faker("word")
    name = factory.Faker("word")
    repository_type = "fake"
    config = factory.Faker("resource_config")
    entry_repo_id = factory.LazyFunction(make_unique_id)
    message = "created"
    created_by = "kristoff@example.com"


class AddEntryFactory(factory.Factory):
    class Meta:
        model = commands.AddEntry

    entity_id = factory.LazyFunction(make_unique_id)
    resource_id = factory.Faker("word")
    entry = {
        "baseform": "bra",
    }
    user = factory.Faker("email")
    message = "added"


class AddEntriesFactory(factory.Factory):
    class Meta:
        model = commands.AddEntries

    resource_id = factory.Faker("word")
    entries = [
        {
            "entry": {
                "baseform": "bra",
            }
        }
    ]
    user = factory.Faker("email")
    message = "added"


class AddEntriesInChunksFactory(AddEntriesFactory):
    class Meta:
        model = commands.AddEntriesInChunks

    chunk_size = 50


class ImportEntriesFactory(AddEntriesFactory):
    class Meta:
        model = commands.ImportEntries

    message = "imported"


class ImportEntriesInChunksFactory(AddEntriesInChunksFactory):
    class Meta:
        model = commands.ImportEntriesInChunks

    message = "imported"


class UpdateEntryFactory(factory.Factory):
    class Meta:
        model = commands.UpdateEntry

    resource_id = factory.Faker("word")
    entry_id = factory.Faker("word")
    entry = {}
    user = factory.Faker("email")
    message = "added"


class DeleteEntryFactory(factory.Factory):
    class Meta:
        model = commands.DeleteEntry

    resource_id = factory.Faker("word")
    entry_id = factory.Faker("word")
    user = factory.Faker("email")
    message = "deleted"



from typing import Callable, Type
from fastapi import Depends
from sqlalchemy.orm import Session
from starlette.requests import Request

from karp.foundation.events import EventBus


def get_eventbus(request: Request) -> EventBus:
    return request.state.container.get(EventBus)
